package com.tecnavia.tabridge;

import android.content.Context;
import android.content.Intent;
import android.content.res.Configuration;
import android.os.Build;
import android.os.Bundle;
import android.util.Log;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.activity.OnBackPressedCallback;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;

import com.facebook.react.ReactDelegate;
import com.facebook.react.ReactNativeHost;
import com.facebook.react.modules.core.PermissionAwareActivity;
import com.facebook.react.modules.core.PermissionListener;
import com.tecnavia.tabridge.broadcasts.TaReceiver;
import com.tecnavia.tabridge.hacks.NativoReflection;
import com.tecnavia.tabridge.listeners.ActivityStarterModuleListener;
import com.tecnavia.tabridge.listeners.TaFragmentDelegate;
import com.tecnavia.tabridge.modules.ActivityStarterModule;

/**
 * RN (0.67.5)
 * Even if exists ReactFragment i make a completely different class
 * because ReactFragment is bad written, specially in the piece of code related to Delegate
 * The reason is that is impossible to override a delegate and return a new react root view and also a react native host
 */
public class TaFragment extends Fragment implements PermissionAwareActivity, ActivityStarterModuleListener {
	public static final String TAG = "TA_FRAGMENT";

	private static final String SI_BUILD_PROPS = "BUILD_PROPS";
	private static final String SI_OPTIONAL_PROPS = "OPTIONAL_PROPS";
	private static final String SI_DELEGATE = "BUILD_DELEGATE";

	private TaReactFragmentDelegate mReactDelegate;

	@Nullable
	private PermissionListener mPermissionListener;

	private TaProps mProps;
	private ActivityStarterModule module;
	private String componentName = "TecnaviaApplication";
	private TaReceiver taReceiver;
	private TaFragmentDelegate delegate;
	private Bundle buildProps = new Bundle();
	private Bundle optionalProps = new Bundle();

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		getReactDelegate().loadApp();
		return getReactDelegate().getReactRootView();
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		if (componentName == null) {
			throw new IllegalStateException("Cannot loadApp if component name is null");
		} else {
			if(savedInstanceState != null){
				buildProps = savedInstanceState.getBundle(SI_BUILD_PROPS);
				optionalProps = savedInstanceState.getBundle(SI_OPTIONAL_PROPS);
			}
			mProps = new TaProps(getActivity());
			mProps.setBuildProps(buildProps);
			mProps.setOptionalProps(optionalProps);
			try {
			    this.mReactDelegate = new TaReactFragmentDelegate(getActivity(), this.getReactNativeHost(), componentName, mProps.getPropsFromIntent(getActivity().getIntent()));
			} catch (Exception e) {
				throw new RuntimeException(e);
			}
		}

		// This callback is only called when MyFragment is at least started
		OnBackPressedCallback callback = new OnBackPressedCallback(true) {
			@Override
			public void handleOnBackPressed() {
				// Handle the back button event
				Log.d(TAG, "back button pressed");
				getReactDelegate().onBackPressed();
			}
		};
		requireActivity().getOnBackPressedDispatcher().addCallback(this, callback);

		NativoReflection.init(getActivity());

		registerReceiver();
		registerModuleListener();
	}

	@Override
	public void onResume() {
		super.onResume();
		NativoReflection.onResume();
		getReactDelegate().onHostResume();
	}

	@Override
	public void onPause() {
		super.onPause();
		NativoReflection.onPause();
		getReactDelegate().onHostPause();
	}

	@Override
	public void onDestroy() {
		super.onDestroy();
		unregisterReceiver();
		getReactDelegate().onHostDestroy();
	}

	@Override
	public void onSaveInstanceState(@NonNull Bundle outState) {
		super.onSaveInstanceState(outState);
		outState.putBundle(SI_BUILD_PROPS, buildProps);
		outState.putBundle(SI_OPTIONAL_PROPS, optionalProps);
	}

	@Override
	public void onConfigurationChanged(@NonNull Configuration newConfig) {
		NativoReflection.onConfigurationChanged(newConfig);
		getResources().getConfiguration().updateFrom( newConfig );
		super.onConfigurationChanged(newConfig);
		getReactNativeHost().getReactInstanceManager().onConfigurationChanged(getActivity(), newConfig);
		Intent intent = new Intent("onConfigurationChanged");
		intent.putExtra("newConfig", newConfig);
		if(getActivity() != null) {
			getActivity().sendBroadcast(intent);
		}
	}

	@Override
	public boolean isActionModeVisible() {
		if(this.delegate != null){
			return this.delegate.isActionModeVisible();
		}
		return false;
	}

	@Override
	public Bundle getInitProps() {
		return null;
	}

	@Override
	public boolean handleClose() {
		if(this.delegate != null){
			return this.delegate.handleClose();
		}
		return false;
	}

	@Override
	public boolean handleOpenUrl(String url) {
		if(this.delegate != null){
			return this.delegate.handleOpenUrl(url);
		}
		return false;
	}

	@Override
	public boolean handleTrackAction(Bundle data) {
		if(this.delegate != null){
			return this.delegate.handleTrackAction(data);
		}
		return false;
	}

	@Override
	public boolean handleTokenExpired(){
		if(this.delegate != null){
			return this.delegate.handleTokenExpired();
		}	
		return false;
	}

	/**
	 * reload JS context
	 */
	@Override
	public void onReload() {
		try {
			if(getActivity() != null) {
				getActivity().runOnUiThread(() -> {
					try {
						TaReactNative.destroy();
						if(this.delegate != null){
							this.delegate.recreate();
						}
					} catch (Exception e) {
						e.printStackTrace();
					}
				});
			}
		} catch (Exception e){
			e.printStackTrace();
		}
	}

	public void onUserInteraction(){
		if(getActivity() != null){
			Intent i = new Intent();
			i.setAction(TaReceiver.BROADCAST_ON_USER_INTERACT);
			getActivity().sendBroadcast(i);
		}
	}

	protected ReactDelegate getReactDelegate() {
		return this.mReactDelegate;
	}

	protected ReactNativeHost getReactNativeHost() {
		if(getActivity() != null) {
			return TaReactNative
					.getInstance()
					.getHost(getActivity().getApplication(), mProps.getVersionName());
		}
		return null;
	}

	/**
	 * attach activity to module listener
	 */
	private void registerModuleListener(){
		try {
			getReactNativeHost().getReactInstanceManager().addReactInstanceEventListener(context -> {
				try {
					module = context.getNativeModule(ActivityStarterModule.class);
					if(module != null) {
						module.setListener(TaFragment.this);
					}
				} catch (Exception e) {
					e.printStackTrace();
				}
			});
		} catch (Exception e){
			e.printStackTrace();
		}
	}

	private void registerReceiver(){
		try{
			if(taReceiver == null) {
				taReceiver = new TaReceiver(getReactNativeHost().getReactInstanceManager());
			}
			if(getActivity() != null) {
				if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
					getActivity().registerReceiver(taReceiver, TaReceiver.getBroadcastFilter(), Context.RECEIVER_NOT_EXPORTED);
				} else {
					getActivity().registerReceiver(taReceiver, TaReceiver.getBroadcastFilter());
				}
			}
		}catch (Exception e){
			e.printStackTrace();
		}
	}

	private void unregisterReceiver(){
		try{
			if(getActivity() != null && taReceiver != null) {
				getActivity().unregisterReceiver(taReceiver);
			}
		}catch (Exception e){
			e.printStackTrace();
		}
	}

	public TaFragment setBuildProps(Bundle props){
		buildProps = props;
		return this;
	}

	public TaFragment setOptionalProps(Bundle props){
		optionalProps = props;
		return this;
	}

	public TaFragment setComponentName(String name){
		componentName = name;
		return this;
	}

	public TaFragment setDelegate(TaFragmentDelegate delegate) {
		this.delegate = delegate;
		return this;
	}

	public void keyDown(int keyCode, KeyEvent event) {
		TaKeyUtils.sendKeysEvent(getActivity(), TaReceiver.BROADCAST_ON_KEY_PRESS_DOWN, event);
	}

	public void keyUp(int keyCode, KeyEvent event) {
		TaKeyUtils.sendKeysEvent(getActivity(), TaReceiver.BROADCAST_ON_KEY_PRESS_UP, event);
	}

	public void handleNewIntent(Intent intent){
		Log.d(TAG, "new intent received");
		try {
			if (mProps.intentNotEmpty(intent)) {
				Bundle props = mProps.getPropsFromIntent(intent);
				Log.d(TAG, "new intent received props: " + props.toString());
				Intent i = new Intent();
				i.setAction(TaReceiver.BROADCAST_ON_NEW_INTENT);
				i.putExtra(TaReceiver.BUNDLE_EXTRA, props);
				getActivity().sendBroadcast(i);
			}
		} catch(Exception e){
			e.printStackTrace();
		}
		try {
			mReactDelegate.getReactInstanceManager().onNewIntent(intent);
		} catch (Exception e){
			e.printStackTrace();
		}
	}

	public void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		this.mReactDelegate.onActivityResult(requestCode, resultCode, data, true);
	}

	public void onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
		super.onRequestPermissionsResult(requestCode, permissions, grantResults);
		if (this.mPermissionListener != null && this.mPermissionListener.onRequestPermissionsResult(requestCode, permissions, grantResults)) {
			this.mPermissionListener = null;
		}
	}

	public int checkPermission(String permission, int pid, int uid) throws NullPointerException {
		return getActivity().checkPermission(permission, pid, uid);
	}

	public int checkSelfPermission(String permission) throws NullPointerException {
		return getActivity().checkSelfPermission(permission);
	}

	public void requestPermissions(String[] permissions, int requestCode, PermissionListener listener) {
		this.mPermissionListener = listener;
		this.requestPermissions(permissions, requestCode);
	}
}
