package com.tecnavia.tabridge.tasks;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.media.RingtoneManager;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import androidx.core.app.NotificationCompat;

import android.util.Log;

import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactContext;
import com.tecnavia.tabridge.R;

public class NotificationTask extends AsyncTask<ReactContext, Void, Void> {
	private static final String TAG = "NotificationTask";

	private final Promise promise;
	private final Bundle notification;
	private final NotificationManager notificationManager;

	public NotificationTask(NotificationManager notificationManager, Bundle notification, Promise promise) {
		this.promise = promise;
		this.notification = notification;
		this.notificationManager = notificationManager;
	}

	@Override
	protected Void doInBackground(ReactContext... contexts) {
		try {
			ReactContext reactContext = contexts[0];
			if (reactContext == null) {
				if (promise != null) {
					promise.reject("notification/display_notification_error", "Could not find main activity class");
				}
				return null;
			}

			String notificationId = notification.getString("message_id");
			Log.d(TAG, "ID: " + notificationId);
			NotificationCompat.Builder nb = null;
			try {
				String channelId = notification.getString("channelId");
				nb = new NotificationCompat.Builder(reactContext, channelId);
			} catch (Exception e) {
				e.printStackTrace();

			}

			if(nb == null){
				if(promise != null){
					promise.reject("notification/display_notification_error", "Could not find notification builder");
				}
				return null;
			}

			if (notification.containsKey("body")) {
				nb = nb.setContentText(notification.getString("body"));
			}

			if (notification.containsKey("data")) {
				nb = nb.setExtras(notification.getBundle("data"));
			}

			if (notification.containsKey("sound")) {
				Uri sound = getSound(reactContext, notification.getString("sound"));
				nb = nb.setSound(sound);
			}

			if (notification.containsKey("subtitle")) {
				nb = nb.setSubText(notification.getString("subtitle"));
			}

			if (notification.containsKey("title")) {
				nb = nb.setContentTitle(notification.getString("title"));
			}

			Bitmap largeIcon = getBitmap(reactContext, "ic_launcher");
			if (largeIcon != null) {
				nb = nb.setLargeIcon(largeIcon);
			}

			// Create the notification intent
			try {
				PendingIntent contentIntent = createIntent(reactContext, notification, notificationId);
				nb = nb.setContentIntent(contentIntent);
			} catch (ClassNotFoundException e){
				e.printStackTrace();
			}
			nb = nb.setSmallIcon(R.drawable.icon_push);
			nb = nb.setVisibility(NotificationCompat.VISIBILITY_PUBLIC);
			nb = nb.setAutoCancel(true);

			// Build the notification and send it
			Notification builtNotification = nb.build();
			notificationManager.notify(notificationId.hashCode(), builtNotification);

			if (promise != null) {
				promise.resolve(null);
			}
		} catch (Exception e) {
			Log.e(TAG, "Failed to send notification", e);
			if (promise != null) {
				promise.reject("notification/display_notification_error", "Could not send notification", e);
			}
		}

		return null;
	}

	private PendingIntent createIntent(Context context, Bundle notification, String notificationId) throws ClassNotFoundException {
		Intent intent = new Intent(context, Class.forName("com.tecnaviaapplication.SplashActivity"));
		intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TASK);
		intent.putExtras(notification);
		return PendingIntent.getActivity(
				context,
				notificationId.hashCode(),
				intent,
				Build.VERSION.SDK_INT >= Build.VERSION_CODES.M ? PendingIntent.FLAG_UPDATE_CURRENT | PendingIntent.FLAG_IMMUTABLE : PendingIntent.FLAG_UPDATE_CURRENT
		);
	}

	private Bitmap getBitmap(ReactContext reactContext, String image) {
		int largeIconResId = getIcon(reactContext, image);
		return BitmapFactory.decodeResource(reactContext.getResources(), largeIconResId);
	}

	private int getIcon(ReactContext reactContext, String icon) {
		int resourceId = getResourceId(reactContext, "mipmap", icon);

		if (resourceId == 0) {
			resourceId = getResourceId(reactContext, "drawable", icon);
		}

		return resourceId;
	}


	private int getResourceId(Context context, String type, String image) {
		return context.getResources().getIdentifier(image, type, context.getPackageName());
	}

	private Uri getSound(Context context, String sound) {
		if (sound == null) {
			return null;
		} else if (sound.contains("://")) {
			return Uri.parse(sound);
		} else if (sound.equalsIgnoreCase("default")) {
			return RingtoneManager.getDefaultUri(RingtoneManager.TYPE_NOTIFICATION);
		} else {
			int soundResourceId = getResourceId(context, "raw", sound);
			if (soundResourceId == 0) {
				soundResourceId = getResourceId(context, "raw", sound.substring(0, sound.lastIndexOf('.')));
			}
			return Uri.parse("android.resource://" + context.getPackageName() + "/" + soundResourceId);
		}
	}
}
